<?php

namespace App\Model;

use Nette\Database\Table\ActiveRow;
use Nette\Database\Table\Selection;


class Group extends Base {

    /**
     * Name of the database table
     * @var string
     */
    protected $tableName = 'group';

    /**
     * @param array $params
     * @return \Nette\Database\Table\Selection
     */
    public function findAllByKw(array $params): Selection
    {
        $records = $this->database->table('group');

        if (isset($params['keywords'])) {
            $records = $records->where("name LIKE ?", "%" . $params['keywords'] . "%");
        }
        if (isset($params['filter']) && $params['filter'] != 'none') {
            $records = $records->where("name LIKE ?", $params['filter'] . "%");
        }
        $records = $records->order($params['sort'] . ' ' . $params['order']);

        return $records;
    }

    /**
     * @param int $groupId
     */
    public function deleteAssociatedRecords(int $groupId): void
    {
        $related = $this->database->table('group_has_publication')->where(["group_id" => $groupId]);
        $related2 = $this->database->table('submitter_has_group')->where(["group_id" => $groupId]);
        $record = $this->database->table('group')->get($groupId);

        foreach ($related as $rel) {
            $rel->delete();
        }

        foreach ($related2 as $rel) {
            $rel->delete();
        }

        if ($record) {
            $record->delete();
        }
    }

    /**
     * @param string $name
     * @return FALSE|\Nette\Database\Table\ActiveRow
     */
    public function findOneByName(string $name)
    {
        return $this->findOneBy([
            'name'  =>  $name
        ]);
    }

    /**
     * @param int $group_id
     * @return Selection
     */
    public function getGroupPublications(int $group_id): Selection
    {
        return $this->database->table('group_has_publication')
            ->select('publication.journal.name AS journal,
                    publication.journal.id AS journal_id,
                    publication.publisher.name AS publisher,
                    publication.conference_year.location AS location, 
                    publication.conference_year.name AS name,
                    publication.conference_year.id AS cy_id,
                    publication.type_of_report AS type, 
                    publication.id, 
                    publication.pub_type, 
                    publication.title, 
                    publication.volume, 
                    publication.number, 
                    publication.pages, 
                    publication.issue_month AS month_eng, 
                    publication.issue_year AS year, 
                    publication.url, 
                    publication.note, 
                    publication.editor, 
                    publication.edition, 
                    publication.address, 
                    publication.howpublished, 
                    publication.chapter, 
                    publication.booktitle, 
                    publication.school,
                    publication.institution, 
                    publication.conference_year_id')
            ->where('group_has_publication.group_id = ?', $group_id);
    }

    /**
     * Returns groups in format for HomepageSearchForm, t.j. array where key is group id and value is group name.
     * @return array
     */
    public function getGroupsForSearchForm(): array
    {
        return $this->getTable()
            ->select('id, name')
            ->fetchPairs('id', 'name');
    }
}
